@@ -5,7 +5,7 @@
 package main
 
 const (
-	version = "0.8.3"
+	version = "0.10.0"
 	// defaults
 	defaultBufferSize    = 100
 	defaultConfigRowSep  = 2
@@ -17,7 +17,6 @@ const (
 	defaultRpadMultiple  = 100
 	defaultRSABits       = 4096
 	defaultShortIDBytes  = 4
-	defaultSleepTime     = 100
 	defaultTimeout       = 10
 	defaultTLSMinVersion = 12
 	defaultTLS10         = 10
@@ -42,6 +41,7 @@ const (
 	oxAltBody        = "This message is encrypted (XEP-0373: OpenPGP for XMPP)."
 	pubsubPubOptions = "http://jabber.org/protocol/pubsub#publish-options"
 	strChat          = "chat"
+	strEmpty         = ""
 	strError         = "error"
 	strGroupchat     = "groupchat"
 	strHeadline      = "headline"
