@@ -15,6 +15,8 @@ import (
 	"net"
 	"os"
 	"os/signal"
+	osUser "os/user"
+	"runtime"
 	"strings"
 	"time"
 
@@ -31,15 +33,12 @@ type configuration struct {
 	alias    string
 }
 
-func closeAndExit(client *xmpp.Client, cancel context.CancelFunc, err error) {
-	// Wait for a short time as some messages are not delivered by the server
-	// if the connection is closed immediately after sending a message.
-	time.Sleep(defaultSleepTime * time.Millisecond)
-	cancel()
+func closeAndExit(client *xmpp.Client, err error) {
 	client.Close()
 	if err != nil {
 		log.Fatal(err)
 	}
+	os.Exit(0)
 }
 
 func readMessage(messageFilePath string) (string, error) {
@@ -94,6 +93,7 @@ func main() {
 		message, user, server, password, alias string
 		oxPrivKey                              *crypto.Key
 		recipients                             []recipientsType
+		fast                                   xmpp.Fast
 	)
 
 	// Define command line flags.
@@ -135,6 +135,7 @@ func main() {
 	flagOOBFile := getopt.StringLong("oob-file", 0, "", "URL to send a file as out of band data.")
 	flagHeadline := getopt.BoolLong("headline", 0, "Send message as type headline.")
 	flagSCRAMPinning := getopt.StringLong("scram-mech-pinning", 0, "", "Enforce the use of a certain SCRAM authentication mechanism.")
+	flagSSDPOff := getopt.BoolLong("ssdp-off", 0, "Disable XEP-0474: SASL SCRAM Downgrade Protection.")
 
 	// Parse command line flags.
 	getopt.Parse()
@@ -146,7 +147,9 @@ func main() {
 		os.Exit(0)
 	case *flagVersion:
 		// If requested, show version and quit.
-		fmt.Println("go-sendxmpp", version)
+		fmt.Println("Go-sendxmpp", version)
+		system := runtime.GOOS + "/" + runtime.GOARCH
+		fmt.Println("System:", system, runtime.Version())
 		fmt.Println("License: BSD-2-clause")
 		os.Exit(0)
 		// Quit if Ox (OpenPGP for XMPP) is requested for unsupported operations like
@@ -169,6 +172,22 @@ func main() {
 		log.Fatal("Can't use message type headline for groupchat messages.")
 	}
 
+	// Print a warning if go-sendxmpp is run by the user root on non-windows systems.
+	if runtime.GOOS != "windows" {
+		// Get the current user.
+		currUser, err := osUser.Current()
+		if err != nil {
+			log.Fatal("Failed to get current user: ", err)
+		}
+		if currUser.Username == "root" {
+			fmt.Println("WARNING: It seems you are running go-sendxmpp as root user.\n" +
+				"This is is not recommended as go-sendxmpp does not require root " +
+				"privileges. Please consider using a less privileged user. For an " +
+				"example how to do this with sudo please consult the manpage chapter " +
+				"TIPS.")
+		}
+	}
+
 	switch *flagSCRAMPinning {
 	case "", "SCRAM-SHA-1", "SCRAM-SHA-1-PLUS", "SCRAM-SHA-256", "SCRAM-SHA-256-PLUS",
 		"SCRAM-SHA-512", "SCRAM-SHA-512-PLUS":
@@ -217,6 +236,22 @@ func main() {
 	if *flagPassword != "" {
 		password = *flagPassword
 	}
+	// If no server part is specified in the username but a server is specified
+	// just assume the server is identical to the server part and hope for the
+	// best. This is for compatibility with the old perl sendxmpp config files.
+	var serverpart string
+	if !strings.Contains(user, "@") && server != "" {
+		// Remove port if server contains it.
+		if strings.Contains(server, ":") {
+			serverpart, _, err = net.SplitHostPort(server)
+			if err != nil {
+				log.Fatal(err)
+			}
+		} else {
+			serverpart = server
+		}
+		user = user + "@" + serverpart
+	}
 
 	switch {
 	// Use "go-sendxmpp" if no nick is specified via config or command line flag.
@@ -230,6 +265,18 @@ func main() {
 	// Timeout
 	timeout := time.Duration(*flagTimeout) * time.Second
 
+	clientID, err := getClientID(user)
+	if err != nil {
+		fmt.Println(err)
+	}
+
+	fast, _ = getFastData(user, password)
+	// Reset FAST token and mechanism if expired.
+	if time.Now().After(fast.Expiry) {
+		fast.Token = ""
+		fast.Mechanism = ""
+	}
+
 	// Use ALPN
 	var tlsConfig tls.Config
 	tlsConfig.ServerName = user[strings.Index(user, "@")+1:]
@@ -250,12 +297,14 @@ func main() {
 		os.Exit(0)
 	}
 
+	resource := "go-sendxmpp." + getShortID()
+
 	// Set XMPP connection options.
 	options := xmpp.Options{
 		Host:        server,
 		User:        user,
 		DialTimeout: timeout,
-		Resource:    "go-sendxmpp." + getShortID(),
+		Resource:    resource,
 		Password:    password,
 		// NoTLS doesn't mean that no TLS is used at all but that instead
 		// of using an encrypted connection to the server (direct TLS)
@@ -263,11 +312,16 @@ func main() {
 		// set when NoTLS is set go-sendxmpp won't use unencrypted
 		// client-to-server connections.
 		// See https://pkg.go.dev/github.com/xmppo/go-xmpp#Options
-		NoTLS:     !*flagDirectTLS,
-		StartTLS:  !*flagDirectTLS,
-		Debug:     *flagDebug,
-		TLSConfig: &tlsConfig,
-		Mechanism: *flagSCRAMPinning,
+		NoTLS:         !*flagDirectTLS,
+		StartTLS:      !*flagDirectTLS,
+		Debug:         *flagDebug,
+		TLSConfig:     &tlsConfig,
+		Mechanism:     *flagSCRAMPinning,
+		SSDP:          !*flagSSDPOff,
+		UserAgentSW:   resource,
+		UserAgentID:   clientID,
+		FastToken:     fast.Token,
+		FastMechanism: fast.Mechanism,
 	}
 
 	// Read message from file.
@@ -312,13 +366,41 @@ func main() {
 	// Connect to server.
 	client, err := connect(options, *flagDirectTLS)
 	if err != nil {
-		log.Fatal(err)
+		if fast.Token != "" {
+			// Reset FAST token and mechanism if FAST login failed.
+			fast.Token = ""
+			fast.Mechanism = ""
+			fast.Expiry = time.Now()
+			err := writeFastData(user, password, fast)
+			if err != nil {
+				fmt.Println(err)
+			}
+			options.FastToken = ""
+			// Try to connect to server without FAST.
+			client, err = connect(options, *flagDirectTLS)
+			if err != nil {
+				log.Fatal(err)
+			}
+		} else {
+			log.Fatal(err)
+		}
 	}
 
+	// Update fast token if a new one is received or expiry time is reduced.
+	if (client.Fast.Token != "" && client.Fast.Token != fast.Token) ||
+		(client.Fast.Expiry.Before(fast.Expiry) && !client.Fast.Expiry.IsZero()) {
+		fast.Token = client.Fast.Token
+		fast.Mechanism = client.Fast.Mechanism
+		fast.Expiry = client.Fast.Expiry
+		err := writeFastData(user, password, fast)
+		if err != nil {
+			fmt.Println(err)
+		}
+	}
 	iqc := make(chan xmpp.IQ, defaultBufferSize)
 	msgc := make(chan xmpp.Chat, defaultBufferSize)
 	ctx, cancel := context.WithCancel(context.Background())
-	go rcvStanzas(client, iqc, msgc, ctx)
+	go rcvStanzas(client, ctx, iqc, msgc)
 	for _, r := range getopt.Args() {
 		var re recipientsType
 		re.Jid = r
@@ -336,7 +418,8 @@ func main() {
 	for i, recipient := range recipients {
 		validatedJid, err := MarshalJID(recipient.Jid)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		recipients[i].Jid = validatedJid
 	}
@@ -345,52 +428,62 @@ func main() {
 	case *flagOxGenPrivKeyX25519:
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		err = oxGenPrivKey(validatedOwnJid, client, iqc, *flagOxPassphrase, "x25519")
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		os.Exit(0)
 	case *flagOxGenPrivKeyRSA:
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		err = oxGenPrivKey(validatedOwnJid, client, iqc, *flagOxPassphrase, "rsa")
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		os.Exit(0)
 	case *flagOxImportPrivKey != "":
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		err = oxImportPrivKey(validatedOwnJid, *flagOxImportPrivKey,
 			client, iqc)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		os.Exit(0)
 	case *flagOxDeleteNodes:
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		err = oxDeleteNodes(validatedOwnJid, client, iqc)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		os.Exit(0)
 	case *flagOx:
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		oxPrivKey, err = oxGetPrivKey(validatedOwnJid, *flagOxPassphrase)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 	}
 
@@ -398,7 +491,8 @@ func main() {
 		message, err = httpUpload(client, iqc, tlsConfig.ServerName,
 			*flagHTTPUpload, timeout)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 	}
 
@@ -408,7 +502,8 @@ func main() {
 		// Check if the URI is valid.
 		uri, err := validURI(message)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 		message = uri.String()
 	}
@@ -431,7 +526,8 @@ func main() {
 				_, err = client.JoinMUCNoHistory(recipient.Jid, alias)
 			}
 			if err != nil {
-				closeAndExit(client, cancel, err)
+				cancel()
+				closeAndExit(client, err)
 			}
 		}
 	}
@@ -443,7 +539,8 @@ func main() {
 		// Send raw XML
 		_, err = client.SendOrg(message)
 		if err != nil {
-			closeAndExit(client, cancel, err)
+			cancel()
+			closeAndExit(client, err)
 		}
 	case *flagInteractive:
 		// Send in endless loop (for usage with e.g. "tail -f").
@@ -454,16 +551,23 @@ func main() {
 		go func() {
 			for range c {
 				cancel()
-				client.Close()
-				os.Exit(0)
+				closeAndExit(client, nil)
 			}
 		}()
 		for {
 			message, err = reader.ReadString('\n')
-			message = strings.TrimSuffix(message, "\n")
 			if err != nil {
-				closeAndExit(client, cancel, errors.New("failed to read from stdin"))
+				select {
+				case <-ctx.Done():
+					return
+				default:
+					if err != nil {
+						cancel()
+						closeAndExit(client, errors.New("failed to read from stdin"))
+					}
+				}
 			}
+			message = strings.TrimSuffix(message, "\n")
 
 			// Remove invalid code points.
 			message = validUTF8(message)
@@ -485,7 +589,8 @@ func main() {
 					}
 					_, err = client.SendOrg(oxMessage)
 					if err != nil {
-						closeAndExit(client, cancel, err)
+						cancel()
+						closeAndExit(client, err)
 					}
 				default:
 					_, err = client.Send(xmpp.Chat{
@@ -493,13 +598,23 @@ func main() {
 						Type:   msgType, Text: message,
 					})
 					if err != nil {
-						closeAndExit(client, cancel, err)
+						cancel()
+						closeAndExit(client, err)
 					}
 				}
 			}
 		}
 	case *flagListen:
 		tz := time.Now().Location()
+		// Quit if ^C is pressed.
+		c := make(chan os.Signal, 1)
+		signal.Notify(c, os.Interrupt)
+		go func() {
+			for range c {
+				cancel()
+				closeAndExit(client, nil)
+			}
+		}()
 		for {
 			v := <-msgc
 			switch {
@@ -604,7 +719,8 @@ func main() {
 				}
 				_, err = client.SendOrg(oxMessage)
 				if err != nil {
-					closeAndExit(client, cancel, err)
+					cancel()
+					closeAndExit(client, err)
 				}
 			default:
 				_, err = client.Send(xmpp.Chat{
@@ -612,10 +728,12 @@ func main() {
 					Type:   msgType, Text: message,
 				})
 				if err != nil {
-					closeAndExit(client, cancel, err)
+					cancel()
+					closeAndExit(client, err)
 				}
 			}
 		}
 	}
-	closeAndExit(client, cancel, nil)
+	cancel()
+	closeAndExit(client, nil)
 }
