@@ -316,75 +316,34 @@ func oxPublishPubKey(jid string, client *xmpp.Client, iqc chan xmpp.IQ, pubKey *
 }
 
 func oxGetPrivKeyLoc(jid string) (string, error) {
-	var err error
-	var homeDir, dataDir string
-	switch {
-	case os.Getenv("$XDG_DATA_HOME") != "":
-		dataDir = os.Getenv("$XDG_DATA_HOME")
-	case os.Getenv("$XDG_HOME") != "":
-		homeDir = os.Getenv("$XDG_HOME")
-		dataDir = homeDir + "/.local/share"
-	case os.Getenv("$HOME") != "":
-		homeDir = os.Getenv("$HOME")
-		dataDir = homeDir + "/.local/share"
-	default:
-		homeDir, err = os.UserHomeDir()
-		if err != nil {
-			return strError, fmt.Errorf("oxGetPrivKeyLoc: failed to determine user dir: %w", err)
-		}
-		if homeDir == "" {
-			return strError, errors.New("oxGetPrivKeyLoc: received empty string for home directory")
-		}
-		dataDir = homeDir + "/.local/share"
-	}
-	dataDir += "/go-sendxmpp/oxprivkeys/"
-	if _, err = os.Stat(dataDir); os.IsNotExist(err) {
-		err = os.MkdirAll(dataDir, defaultDirRights)
-		if err != nil {
-			return strError, fmt.Errorf("oxGetPrivKeyLoc: could not create folder for private keys: %w", err)
-		}
+	dataDir, err := getDataPath("oxprivkeys/")
+	if err != nil {
+		return strError, fmt.Errorf("oxGetPubKeyLoc: %w", err)
 	}
 	// TODO: Remove handling of oldDataFile in a later version when it's very likely that there are no
 	// more versions in use using the oldDataFile (<0.8.3).
 	oldDataFile := dataDir + base64.StdEncoding.EncodeToString([]byte(jid))
-	dataFile := dataDir + strings.Replace(jid, "@", "_at_", -1)
+	oldDataFile2 := dataDir + strings.Replace(jid, "@", "_at_", -1)
+	dataFile := dataDir + strings.Replace(strings.Replace(jid, "@", "_at_", -1), ".", "_", -1)
 	if _, err := os.Stat(oldDataFile); err == nil {
 		err := os.Rename(oldDataFile, dataFile)
 		if err != nil {
 			return dataFile, err
 		}
 	}
+	if _, err := os.Stat(oldDataFile2); err == nil {
+		err := os.Rename(oldDataFile2, dataFile)
+		if err != nil {
+			return dataFile, err
+		}
+	}
 	return dataFile, nil
 }
 
 func oxGetPubKeyLoc(fingerprint string) (string, error) {
-	var err error
-	var homeDir, dataDir string
-	switch {
-	case os.Getenv("$XDG_DATA_HOME") != "":
-		dataDir = os.Getenv("$XDG_DATA_HOME")
-	case os.Getenv("$XDG_HOME") != "":
-		homeDir = os.Getenv("$XDG_HOME")
-		dataDir = homeDir + "/.local/share"
-	case os.Getenv("$HOME") != "":
-		homeDir = os.Getenv("$HOME")
-		dataDir = homeDir + "/.local/share"
-	default:
-		homeDir, err = os.UserHomeDir()
-		if err != nil {
-			return strError, fmt.Errorf("oxGetPubKeyLoc: failed to determine user dir: %w", err)
-		}
-		if homeDir == "" {
-			return strError, errors.New("oxGetPubKeyLoc: received empty string for home directory")
-		}
-		dataDir = homeDir + "/.local/share"
-	}
-	dataDir += "/go-sendxmpp/oxpubkeys/"
-	if _, err = os.Stat(dataDir); os.IsNotExist(err) {
-		err = os.MkdirAll(dataDir, defaultDirRights)
-		if err != nil {
-			return strError, fmt.Errorf("oxGetPubKeyLoc: could not create folder for public keys: %w", err)
-		}
+	dataDir, err := getDataPath("oxpubkeys/")
+	if err != nil {
+		return strError, fmt.Errorf("oxGetPubKeyLoc: %w", err)
 	}
 	dataFile := dataDir + fingerprint
 	return dataFile, nil
