@@ -6,9 +6,12 @@ package main
 
 import (
 	"context"
+	"errors"
 	"fmt"
+	"io"
 	"log"
 	"runtime"
+	"time"
 
 	"github.com/beevik/etree"  // BSD-2-clause
 	"github.com/xmppo/go-xmpp" // BSD-3-Clause
@@ -17,14 +20,18 @@ import (
 func sendIQ(client *xmpp.Client, iqc chan xmpp.IQ, target string, iQtype string, content string) (xmpp.IQ, error) {
 	var iq xmpp.IQ
 	id := getID()
-	c := make(chan xmpp.IQ)
+	c := make(chan xmpp.IQ, defaultBufferSize)
 	go getIQ(id, c, iqc)
 	_, err := client.RawInformation(client.JID(), target, id,
 		iQtype, content)
 	if err != nil {
 		return iq, fmt.Errorf("sendIQ: failed to send iq: %w", err)
 	}
-	iq = <-c
+	select {
+	case iq = <-c:
+	case <-time.After(60 * time.Second):
+		return iq, errors.New("sendIQ: server didn't reply to IQ: " + content)
+	}
 	return iq, nil
 }
 
@@ -38,29 +45,49 @@ func getIQ(id string, c chan xmpp.IQ, iqc chan xmpp.IQ) {
 	}
 }
 
-func rcvStanzas(client *xmpp.Client, iqc chan xmpp.IQ, msgc chan xmpp.Chat, ctx context.Context) {
+func rcvStanzas(client *xmpp.Client, ctx context.Context, iqc chan xmpp.IQ, msgc chan xmpp.Chat) {
+	var received interface{}
+	r := make(chan interface{}, defaultBufferSize)
+	e := make(chan error, defaultBufferSize)
+	go func() {
+		for {
+			select {
+			case <-ctx.Done():
+				return
+			default:
+			}
+			rcv, err := client.Recv()
+			if err != nil {
+				e <- err
+			} else {
+				r <- rcv
+			}
+		}
+	}()
 	for {
-		received, err := client.Recv()
-		// Don't print errors if the program is getting shut down,
-		// as the errors might be triggered from trying to read from
-		// a closed connection.
 		select {
 		case <-ctx.Done():
 			return
-		default:
+		case err := <-e:
 			if err != nil {
-				log.Println(err)
+				if err != io.EOF {
+					closeAndExit(client, err)
+					return
+				}
+				return
 			}
+		case received = <-r:
 		}
 		switch v := received.(type) {
 		case xmpp.Chat:
+
 			msgc <- v
 		case xmpp.IQ:
 			switch v.Type {
 			case "get":
 				var xmlns *etree.Attr
 				iq := etree.NewDocument()
-				err = iq.ReadFromBytes(v.Query)
+				err := iq.ReadFromBytes(v.Query)
 				if err != nil {
 					log.Println("Couldn't parse IQ:",
 						string(v.Query), err)
@@ -69,6 +96,9 @@ func rcvStanzas(client *xmpp.Client, iqc chan xmpp.IQ, msgc chan xmpp.Chat, ctx
 				if query != nil {
 					xmlns = query.SelectAttr("xmlns")
 				}
+				if xmlns == nil {
+					break
+				}
 				switch xmlns.Value {
 				case nsDiscoInfo:
 					root := etree.NewDocument()
